<?php

namespace SGalinski\SgMail\Updates;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgMail\Service\BackendService;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Install\Updates\AbstractUpdate;

/**
 * Migrate template db entries to the correct root pages
 */
class MigrateData extends AbstractUpdate {

	protected $identifier = '';
	/**
	 * @var string
	 */
	protected $title = 'Find all templates & queue entries without site root and assign the correct site root id';

	/**
	 * @var \TYPO3\CMS\Extbase\Object\ObjectManager
	 */
	protected $objectManager;

	/**
	 * @var array
	 */
	protected $tables = [
		'tx_sgmail_domain_model_mail', 'tx_sgmail_domain_model_template'
	];

	/**
	 * Checks whether updates are required.
	 *
	 * @param string &$description The description for the update
	 * @return bool Whether an update is required (TRUE) or not (FALSE)
	 */
	public function checkForUpdate(&$description) {
		$description = 'Set the site_root_id for all the queue entries and templates if not set or pid = 0';
		$oldRowsFound = FALSE;

		foreach ($this->tables as $table) {
			$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($table);
			$queryBuilder->getRestrictions()->removeAll();
			$rowCount = $queryBuilder->select('*')
				->from($table)
				->where(
					$queryBuilder->expr()->eq('site_root_id', 0)
				)->execute()->rowCount();
			if ($rowCount > 0) {
				$oldRowsFound = TRUE;
				break;
			}
		}

		return !(!$oldRowsFound || $this->isWizardDone());
	}

	/**
	 * Performs the accordant updates.
	 *
	 * @param array &$dbQueries Queries done in this update
	 * @param mixed &$customMessages Custom messages
	 * @return bool Whether everything went smoothly or not
	 */
	public function performUpdate(array &$dbQueries, &$customMessages) {
		foreach ($this->tables as $table) {
			$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($table);
			$queryBuilder->getRestrictions()->removeAll();
			$rows = $queryBuilder->select('uid', 'pid')
				->from($table)
				->where(
					$queryBuilder->expr()->eq('site_root_id', 0)
				)->execute()->fetchAll();
			$dbQueries[] = $queryBuilder->getSQL();

			/** @var array $result */
			foreach ($rows as $row) {
				$siteRootId = BackendService::getSiteRoot($row[1]);
				if ($siteRootId === NULL) {
					$this->retrieveFirstSiteRoot();
				}
				$queryBuilder->update($table)
					->where(
						$queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter($row[0]))
					)
					->set('site_root_id', (int) $siteRootId);
				$dbQueries[] = $queryBuilder->getSQL();
			}
		}

		$this->markWizardAsDone();
		return TRUE;
	}

	/**
	 * Get the first site root that is not pid = 0
	 *
	 * @return int
	 */
	private function retrieveFirstSiteRoot() {
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
		$queryBuilder->getRestrictions()->removeAll();
		$pagesResult = $queryBuilder->select('uid')
			->from('pages')
			->where(
				$queryBuilder->expr()->andX(
					$queryBuilder->expr()->eq('is_siteroot', 1),
					$queryBuilder->expr()->neq('uid', 0)
				)
			)
			->setMaxResults(1)
			->execute()->fetch();
		$dbQueries[] = $queryBuilder->getSQL();

		if (empty($pagesResult)) {
			return 0;
		}

		return $pagesResult[0][0];
	}
}
