<?php

namespace SGalinski\SgMail\Updates;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Install\Updates\AbstractUpdate;

/**
 * Update the correct sending times (if any)
 */
class UpdateSendingTimes extends AbstractUpdate {
	/**
	 * @var string
	 */
	protected $identifier = 'tx_sgmail_update_sending_times';

	/**
	 * @var string
	 */
	protected $title = 'Find all sent mails and set the sending time to the tstamp field';

	/**
	 * @var \TYPO3\CMS\Extbase\Object\ObjectManager
	 */
	protected $objectManager;

	/**
	 * @var string
	 */
	protected $table = 'tx_sgmail_domain_model_mail';

	/**
	 * Checks whether updates are required.
	 *
	 * @param string &$description The description for the update
	 * @return bool Whether an update is required (TRUE) or not (FALSE)
	 */
	public function checkForUpdate(&$description) {
		$description = 'Update the sending times if mails have been sent';

		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($this->table);
		$queryBuilder->getRestrictions()->removeAll();
		$rowCount = $queryBuilder->select('*')
			->from($this->table)
			->where(
				$queryBuilder->expr()->andX(
					$queryBuilder->expr()->gt('sending_time', 0),
					$queryBuilder->expr()->eq('last_sending_time', 0)
				)
			)
			->execute()->rowCount();
		if ($rowCount > 0) {
			return TRUE;
		}

		return !(!FALSE || $this->isWizardDone());
	}

	/**
	 * Performs the according updates.
	 *
	 * @param array &$dbQueries Queries done in this update
	 * @param mixed &$customMessages Custom messages
	 * @return bool Whether everything went smoothly or not
	 */
	public function performUpdate(array &$dbQueries, &$customMessages) {
		$dbQueries = [];
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($this->table);
		$queryBuilder->getRestrictions()->removeAll();
		$result = $queryBuilder->select('uid', 'sending_time')
			->from($this->table)
			->where(
				$queryBuilder->expr()->andX(
					$queryBuilder->expr()->gt('sending_time', 0),
					$queryBuilder->expr()->eq('last_sending_time', 0)
				)
			)
			->execute()->fetchAll();
		$dbQueries[] = $queryBuilder->getSQL();

		/** @var array $result */
		foreach ($result as $row) {
			$queryBuilder->update($this->table)
				->where(
					$queryBuilder->expr()->eq('uid', $queryBuilder->createNamedParameter($row['uid'], \PDO::PARAM_INT))
				)
				->set('last_sending_time', (int) $row['sending_time'])
				->execute();
			$dbQueries[] = $queryBuilder->getSQL();
		}

		$this->markWizardAsDone();
		return TRUE;
	}
}
